import 'package:clean_framework/clean_framework_tests.dart';
import 'package:clean_framework_example/example_feature/ui/example_feature_widget.dart';
import 'package:clean_framework_example/example_feature/ui/example_presenter.dart';
import 'package:clean_framework_example/features.dart';
import 'package:clean_framework_example/providers.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  testWidgets(
      'ExampleFeatureWidget hidden, then visible with load, then hide again',
      (tester) async {
    final featureTester = FeatureTester<FeatureState>(featureStatesProvider);

    final testWidget = MaterialApp(
        home: Column(
      children: [
        ExampleFeatureWidget(),
        ElevatedButton(
          key: Key('loadButton'),
          child: Text('load'),
          onPressed: () {
            featureTester.featuresMap.load({
              'features': [
                {'name': 'example', 'version': '1.0', 'state': 'ACTIVE'},
              ]
            });
          },
        ),
        ElevatedButton(
          key: Key('hideButton'),
          child: Text('hide'),
          onPressed: () {
            featureTester.featuresMap.append({
              'features': [
                {'name': 'example', 'version': '1.0', 'state': 'HIDDEN'},
              ]
            });
          },
        )
      ],
    ));

    await featureTester.pumpWidget(tester, testWidget);

    expect(find.byType(ExampleFeatureWidget), findsOneWidget);
    expect(find.byType(ExamplePresenter), findsNothing);
    expect(find.byKey(Key('hidden')), findsOneWidget);

    await tester.tap(find.byKey(Key('loadButton')));
    await tester.pump();

    expect(find.byType(ExamplePresenter), findsOneWidget);
    expect(find.byKey(Key('hidden')), findsNothing);

    await tester.tap(find.byKey(Key('hideButton')));
    await tester.pump();

    expect(find.byType(ExamplePresenter), findsNothing);
    expect(find.byKey(Key('hidden')), findsOneWidget);

    featureTester.dispose();
  });
}
