import 'package:clean_framework/clean_framework.dart';
import 'package:clean_framework/clean_framework_defaults.dart';
import 'package:clean_framework/clean_framework_tests.dart';
import 'package:clean_framework/src/defaults/feature_state/feature_state_rest_loader.dart';

import 'package:flutter_test/flutter_test.dart';
import 'package:mocktail/mocktail.dart';

void main() {
  final featureStateProvider =
      FeatureStateProvider<FeatureState, DefaultFeatureMapper>(
          (_) => DefaultFeatureMapper());

  test('FeatureStateService get success', () async {
    final featureTester = FeatureTester<FeatureState>(featureStateProvider);

    final serviceMock =
        FeatureStateServiceMock.success(featureTester.featuresMap);

    final loader =
        FeatureStateRestLoader(service: serviceMock, onServiceFailure: (_) {});

    await loader.fetchInitialData();

    expect(featureTester.featuresMap.getStateFor(Feature(name: 'login')),
        FeatureState.visible);

    featureTester.dispose();
  });
}

class FeatureStateServiceMock extends Mock implements FeatureStateService {
  FeatureStateServiceMock();

  factory FeatureStateServiceMock.success(FeatureMapper? mapper) {
    final mock = FeatureStateServiceMock();
    when(() => mock.request(requestModel: any(named: 'requestModel')))
        .thenAnswer((_) async {
      mapper?.load({
        'features': [
          {'name': 'login', 'version': '1.0', 'state': 'VISIBLE'},
        ]
      });
      return Right(EmptyJsonResponseModel());
    });
    return mock;
  }
}
