import 'package:clean_framework/clean_framework.dart';
import 'package:clean_framework/clean_framework_defaults.dart';
import 'package:clean_framework/clean_framework_tests.dart';

import 'package:flutter_test/flutter_test.dart';

void main() {
  final featureStateProvider =
      FeatureStateProvider<FeatureState, DefaultFeatureMapper>(
          (_) => DefaultFeatureMapper());

  test('FeatureStateService get success', () async {
    final featureTester = FeatureTester<FeatureState>(featureStateProvider);

    final restApiMock = RestApiMock<String>(
      responseType: RestResponseType.success,
      content: '''
        {
          "features": [
            {"name": "login", "state": "VISIBLE"}
          ]
        }
      ''',
    );

    final service = FeatureStateService(
      restApi: restApiMock,
      featuresMap: featureTester.featuresMap,
    );

    final response = await service.request();
    final model = response.fold((_) {}, (m) => m);

    expect(model, EmptyJsonResponseModel());
    expect(featureTester.featuresMap.getStateFor(Feature(name: 'login')),
        FeatureState.visible);

    featureTester.dispose();
  });
}
