import 'dart:typed_data';

import 'dart:convert';

import 'package:clean_framework/clean_framework.dart';
import 'package:clean_framework/clean_framework_defaults.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:http/http.dart';

void main() {
  test('SimpleRestApi success get', () async {
    final client = SimpleRestApi(
        baseUrl: 'http://fake',
        httpClient: ClientMock()); // Default constructor uses Mockey baseUrl

    final response = await client.request(
      method: RestMethod.get,
      path: 'api/mobile/2.2/features',
    );

    expect(response, allOf(isNotNull, isA<RestResponse>()));
    expect(response.type, RestResponseType.success);
    expect(response.content, '{"Content-type":"application/json"}');

    //TODO Allow dependency injection later to make this testable
  }, skip: true);

  test('SimpleRestApi failure get', () async {
    final client = SimpleRestApi(baseUrl: 'http://fake');

    final response = await client.request(
      method: RestMethod.get,
      path: 'api/mobile/2.2/features',
    );

    expect(response, allOf(isNotNull, isA<RestResponse>()));
    expect(response.type, RestResponseType.unknown);
    expect(response.content.isEmpty, isTrue);
  });

  test('SimpleRestApi post body encoding with double', () async {
    final api = SimpleRestApi(baseUrl: 'http://fake', httpClient: ClientMock());

    final response = await api.request(
        method: RestMethod.post,
        path: 'api/mobile/2.2/features',
        requestBody: {
          'numberField': 123.0,
        });

    expect(response, allOf(isNotNull, isA<RestResponse>()));
    expect(response.type, RestResponseType.success);
    expect(response.content, '{"numberField":123.0}');
  });
}

// This simple mock is not complete, add the rest of the behavior if needed.
class ClientMock implements Client {
  @override
  void close() {}

  @override
  Future<Response> delete(Uri url,
      {Map<String, String>? headers, Object? body, Encoding? encoding}) {
    throw UnimplementedError();
  }

  @override
  Future<Response> get(Uri url, {Map<String, String>? headers}) {
    return Future.value(Response(json.encode(headers), 200));
  }

  @override
  Future<Response> head(Uri url, {Map<String, String>? headers}) {
    throw UnimplementedError();
  }

  @override
  Future<Response> patch(Uri url,
      {Map<String, String>? headers, Object? body, Encoding? encoding}) {
    throw UnimplementedError();
  }

  @override
  Future<Response> post(Uri url,
      {Map<String, String>? headers, Object? body, Encoding? encoding}) {
    return Future.value(Response(body as String, 200));
  }

  @override
  Future<Response> put(Uri url,
      {Map<String, String>? headers, Object? body, Encoding? encoding}) {
    throw UnimplementedError();
  }

  @override
  Future<String> read(Uri url, {Map<String, String>? headers}) {
    throw UnimplementedError();
  }

  @override
  Future<Uint8List> readBytes(Uri url, {Map<String, String>? headers}) {
    throw UnimplementedError();
  }

  @override
  Future<StreamedResponse> send(BaseRequest request) {
    throw UnimplementedError();
  }
}
